/// @addtogroup auxil
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file Functor.h
///
/// @author Marian Mrozek
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2006 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#if !defined(_FUNCTOR_H_)
#define _FUNCTOR_H_

template<class Op2, class Op1>
class ComposedFunctor{
  Op1 op1;
  Op2 op2;
  public:
  ComposedFunctor(const Op2& A_op2, const Op1& A_op1):op1(A_op1),op2(A_op2){}
  typedef typename Op1::argumentType argumentType;
  typedef typename Op2::valueType valueType;
  valueType operator()(argumentType A_a){
    return op2(op1(A_a));
  }
};

template<class Op2, class Op1>
inline ComposedFunctor<Op2,Op1> operator*(const Op2& op2,const Op1& op1){
  return ComposedFunctor<Op2,Op1>(op2,op1);
}

template<typename valType,typename argType>
class Functor{

  public:
  typedef argType argumentType;
  typedef valType valueType;
  typedef valueType (*functionPtrType)(argumentType A_a);
  Functor(valueType (*A_fPtr)(argumentType)):functionPtr(A_fPtr){}
  valueType operator()(argumentType A_a){
    return (*functionPtr)(A_a);
  }
  private:
  valueType (*functionPtr)(argumentType A_a);
};

#endif //_FUNCTOR_H_
/// @}

