/// @addtogroup auxil
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file Stopwatch.h
///
/// @author Marian Mrozek
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2006 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _STOPWATCH_H_
#define _STOPWATCH_H_

#include <sys/time.h>
#include "auxil/clock.h"
#ifdef LINUX
#include <sys/times.h>
class Stopwatch{
  private:
    long double startWorldTime,startProcessTime;
    long double getWorldTime() const{
      struct timeval buffer;
      struct timezone tz;
      gettimeofday(&buffer,&tz);
      return (long double)(buffer.tv_sec)+(long double)(buffer.tv_usec)/1000000;
    }
    long double getProcessTime() const{
      struct tms buffer;
      times(&buffer);
      return (long double)(buffer.tms_utime+buffer.tms_stime) /60.0;
    }
  public:
    Stopwatch():startWorldTime(getWorldTime()),startProcessTime(getProcessTime()){}
    double worldTime() const{ return (double)(getWorldTime()-startWorldTime);}
    double processTime() const{
      long double pt=getProcessTime();
      double ptd=(double)(pt-startProcessTime);
      return ptd;
    }
    friend std::ostream& operator<<(std::ostream& out, const Stopwatch& sw){
      double wt=sw.worldTime();
      double pt= sw.processTime();
      out   << pt << " proc secs (" << wt << " world secs)";
      return out;
    }
};
#else
class Stopwatch{
  private:
    time_t startWorldTime;
    double startProcessTime;
    time_t getWorldTime() const{
      return time(0);
    }
    long double getProcessTime() const{
      return Clock();
    }
  public:
    Stopwatch():startWorldTime(getWorldTime()),startProcessTime(getProcessTime()){}
    double worldTime() const{ return (double)difftime(time(0),startWorldTime);}
    double processTime() const{
      double t=getProcessTime()-startProcessTime;
      return (t>0 ? t : 0.0);
    }
    friend std::ostream& operator<<(std::ostream& out, const Stopwatch& sw){
      out << sw.processTime() << " proc secs (" << sw.worldTime() << " world secs)" ;
      return out;
    }
};
#endif

#endif //  _STOPWATCH_H_
/// @}

