/// @addtogroup bitSet
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file EuclBitSet_PointIterator.h
///
/// @author Marian Mrozek
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2006 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#if !defined(_EUCLBITSET_POINTITERATOR_H_)
#define _EUCLBITSET_POINTITERATOR_H_

/*=======================================================================*/
// PointIterator is a BitSetT iterator
// intended for fast scan of the sets point after point

template <typename P_BitSet,int dim>
class EuclBitSetT<P_BitSet,dim>::PointIterator : public EuclBitSetT<P_BitSet,dim>::BitIterator{
//  friend class BitSetT<P_Bitmap>;
public:
  typedef typename EuclBitSetT<P_BitSet,dim>::BitIterator BitIterator;
  explicit PointIterator(const EuclBitSetT<P_BitSet,dim>& s):BitIterator(s){
    this->findPoint();
  }
  PointIterator(const BitIterator& A_it):BitIterator(A_it){
    this->findPoint();
  }

  PointIterator& operator++(){
    this->BitIterator::operator++();
    this->findPoint();
    return *this;
  }
};

/*=======================================================================*/

#endif
/// @}

