/// @addtogroup interval
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file rounding.cpp
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#include "interval/rounding.h"

#define ROUND_MASK    NOT 0C00h
#define ROUND_NEAREST 0
#define ROUND_DOWN    1 SHL 10
#define ROUND_UP      2 SHL 10
#define ROUND_CUT     3 SHL 10
#define IM_SET        0001h
#define IM_CLEAR      NOT 0001h

#if !defined(BORLAND) && !defined(SUN_OS) && !defined(__ppc__) && !defined(__powerpc__)
//#if defined(LINUX) || defined(WINDOWS_GNU)
// LINUX code designed by Mikolaj Zalewski

  void round_nearest(void){
    short s;
    asm volatile (
      "fstcw %0\n\t"
      "andw $0xf3ff, %1\n\t"
      "orw $0x0000 , %1\n\t"
      "fldcw %1"
      :"=m" (s)
      :"m" (s)
    );
  }

  void round_down(void){
    short s;
    asm volatile (
      "fstcw %0\n\t"
      "andw $0xf3ff, %1\n\t"
      "orw $0x0400 , %1\n\t"
      "fldcw %1"
      :"=m" (s)
      :"m" (s)
    );
  }

  void round_up(void){
    short s;
    asm volatile (
      "fstcw %0\n\t"
      "andw $0xf3ff, %1\n\t"
      "orw $0x0800 , %1\n\t"
      "fldcw %1"
      :"=m" (s)
      :"m" (s)
    );
  }

  void round_cut(void){
    short s;
    asm volatile (
      "fstcw %0\n\t"
      "andw $0xf3ff, %1\n\t"
      "orw $0x0C00 , %1\n\t"
      "fldcw %1"
      :"=m" (s)
      :"m" (s)
    );
  }

  //needed under WINDOWS
  void set_intel_im(void){
    unsigned short s;
    asm volatile (
      "fstcw %0\n\t"
      "orw $0x0001 , %1\n\t"
      "fldcw %1"
      :"=m" (s)
      :"m" (s)
    );
  }

  void clear_intel_im(void){
    unsigned short s;
    asm volatile (
      "fstcw %0\n\t"
      "andw $0xfffe , %1\n\t"
      "fldcw %1"
      :"=m" (s)
      :"m" (s)
    );
  }

#elif defined(__ppc__) || defined(__powerpc__)

  // The following assembly code for PowerPC (Mac) provided by Zin Arai.

  void round_nearest(void){
    asm volatile ("mtfsfi 7,0");
  }

  void round_down(void){
    asm volatile ("mtfsfi 7,3");
  }

  void round_up(void){
    asm volatile ("mtfsfi 7,2");
  }

  void round_cut(void){
    asm volatile ("mtfsfi 7,1");
  }

#elif defined(IBM)

void round_up(void){
  unsigned short fpu_up_cw;
                      asm              fstcw  fpu_up_cw;
                      asm              and    fpu_up_cw, ROUND_MASK;
                      asm              or     fpu_up_cw, ROUND_UP;
                      asm              fldcw  fpu_up_cw;
}

void round_down(void){
  unsigned short fpu_down_cw;
                      asm              fstcw  fpu_down_cw;
                      asm              and    fpu_down_cw, ROUND_MASK;
                      asm              or     fpu_down_cw, ROUND_DOWN;
                      asm              fldcw  fpu_down_cw;
}

void round_nearest(void){
  unsigned short fpu_nearest_cw;
                      asm              fstcw  fpu_nearest_cw;
                      asm              and    fpu_nearest_cw, ROUND_MASK;
                      asm              or     fpu_nearest_cw, ROUND_NEAREST;
                      asm              fldcw  fpu_nearest_cw;
}

void round_cut(void){
  unsigned short fpu_cut_cw;
                      asm              fstcw  fpu_cut_cw;
                      asm              and    fpu_cut_cw, ROUND_MASK;
                      asm              or     fpu_cut_cw, ROUND_CUT;
                      asm              fldcw  fpu_cut_cw;
}

void set_intel_im(void){
  unsigned short fpu_cut_cw;
                      asm              fstcw  fpu_cut_cw;
                      asm              or     fpu_cut_cw, IM_SET;
                      asm              fldcw  fpu_cut_cw;
}
void clear_intel_im(void){
  unsigned short fpu_cut_cw;
                      asm              fstcw  fpu_cut_cw;
                      asm              and    fpu_cut_cw, IM_CLEAR;
                      asm              fldcw  fpu_cut_cw;
}
#endif


int rounding_test(void){
  double a,b,nexta,preva,nextb,eps;
  int result=-1;

  nexta=a=1.5;
  do{
    preva=a;
    a=nexta;
    nexta=(a-1.0)/2+1.0;
  }while(a!=nexta);

  nextb=-1.5;
  do{
    b=nextb;
    nextb=(b+1.0)/2-1.0;
  }while(b!=nextb);

  eps=((2*(preva-1))/3+1);
  if(a == 1 && b == -1 && eps == 1){
    result=ROUNDING_MODE_CUT;
  }else{
    if(a>1  && (preva-1)/3>0){
      result=ROUNDING_MODE_UP;
    }else{
      if(b<-1){
        result=ROUNDING_MODE_DOWN;
      }else{
        result=ROUNDING_MODE_NEAREST;
      }
    }
  }

  return result;

}

void init_fpunit(void){
  #if(defined IBM) // this could also be implemented for LINUX and SPARC but so far was not needed
    // For some reason Win98 turns off some of the error handling of INTEL processors,
    // so we first turn on all numerical error handling
//    #ifndef WINDOWS_GNU          // Windows and Borland C++
    #if defined(BORLAND)          // Windows and Borland C++
       asm              fninit;
    #else                        // Windows and GNU C++
       asm volatile (
          "fninit\n\t"
      );
    #endif
    // next we turn on/off "forbidden numeric operation" error handling for INTEL processors
    #if(defined (INTEL_IM_CLEAR))
      clear_intel_im();
    #else
      set_intel_im();
    #endif
  #endif

  //finally we turn on rounding up, if necessary
  #ifdef UP_ROUNDING
    round_up();
  #endif
}

/// @}
