/// @addtogroup vectalg
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file vector.cpp
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#include <iostream>
#include "vectalg/dimension.h"
#include "vectalg/vector.h"

template class Vector<double,DIM>;

template Vector<double,DIM> abs<double,DIM> (const Vector<double,DIM> &v);
template Vector<double,DIM> operator- <double,DIM> (const Vector<double,DIM> &v);
template Vector<double,DIM> operator+ <double,DIM> (const Vector<double,DIM> &v1,const Vector<double,DIM> &v2);
template Vector<double,DIM> operator- <double,DIM> (const Vector<double,DIM> &v1,const Vector<double,DIM> &v2);
template double operator* <double,DIM> (const Vector<double,DIM> &v1,const Vector<double,DIM> &v2);

template Vector<double,DIM> operator* <double,double,DIM> (const Vector<double,DIM> &v,const double &s);
template Vector<double,DIM> operator* <double,double,DIM> (const double &s,const Vector<double,DIM> &v);
template Vector<double,DIM> operator/ <double,double,DIM> (const Vector<double,DIM> &v, const double &s);

template Vector<double,DIM> operator* <double,long,DIM>(const Vector<double,DIM> &v,const long &s);
template Vector<double,DIM> operator* <double,long,DIM>(const long &s,const Vector<double,DIM> &v);
template Vector<double,DIM> operator/ <double,long,DIM>(const Vector<double,DIM> &v, const long &s);

template Vector<double,DIM> operator* <double,int,DIM>(const Vector<double,DIM> &v,const int &s);
template Vector<double,DIM> operator* <double,int,DIM>(const int &s,const Vector<double,DIM> &v);
template Vector<double,DIM> operator/ <double,int,DIM>(const Vector<double,DIM> &v, const int &s);

template Vector<double,DIM> operator+ <double,DIM> (const Vector<double,DIM> &v,const double &s);
template Vector<double,DIM> operator- <double,DIM> (const Vector<double,DIM> &v,const double &s);
template bool operator < <double,DIM> (const Vector<double,DIM> &v1,const Vector<double,DIM> &v2);
template bool operator > <double,DIM> (const Vector<double,DIM> &v1,const Vector<double,DIM> &v2);
template bool operator<= <double,DIM> (const Vector<double,DIM> &v1,const Vector<double,DIM> &v2);
template bool operator>= <double,DIM> (const Vector<double,DIM> &v1,const Vector<double,DIM> &v2);
template std::ostream &operator<< <double,DIM> (std::ostream &out, const Vector<double,DIM> &v);
template std::istream &operator>> <double,DIM>(std::istream &inp, Vector<double,DIM> &v);

/// @}
